use anyhow::{anyhow, Result};
use binary_reader::BinaryReader;
use chrono::{NaiveDate, NaiveDateTime};

use super::models::Version;

pub fn decode_icf_datetime(rd: &mut BinaryReader) -> Result<NaiveDateTime> {
    let datetime = NaiveDate::from_ymd_opt(
        rd.read_i16()? as i32,
        rd.read_u8()? as u32,
        rd.read_u8()? as u32,
    )
    .ok_or(anyhow!("Invalid date"))?
    .and_hms_milli_opt(
        rd.read_u8()? as u32,
        rd.read_u8()? as u32,
        rd.read_u8()? as u32,
        rd.read_u8()? as u32,
    )
    .ok_or(anyhow!("Invalid time"))?;

    Ok(datetime)
}

pub fn decode_icf_version(rd: &mut BinaryReader) -> Result<Version> {
    let version = Version {
        build: rd.read_u8()?,
        minor: rd.read_u8()?,
        major: rd.read_u16()?,
    };

    Ok(version)
}
